open Mlpost
open Metapost
open Command
open Picture
open Path
open Num
open Num.Infix
open Helpers
open Box

(****
let pen = Pen.transform [Transform.scaled (bp 1.5)] Pen.default

let tabular l =
  "\\begin{tabular}{c}" ^
    String.concat "\\\\\n" l ^
    "\\end{tabular}"

let rec listn n x = if n = 0 then [] else x :: listn (n-1) x

let script s = tex ~dx:zero ~dy:zero ("\\scriptsize\\tt " ^ s)

let textbox ?fill ?(width=bp 8.) ?(padding=false) ?(n=12) ?head ?tail () = 
  let head = match head with None -> [] | Some s -> [script s] in
  let tail = match tail with None -> [] | Some s -> [script s] in
  let d = rect ~pen ~dy:zero (empty ~width ~height:(bp 0.) ()) in
  let d = 
    if padding then hbox ~dx:zero ~dy:zero [empty ~width:(bp 3.) (); d] else d 
  in
  round_rect ?fill (vbox ~pos:`Left ~padding:(bp 2.) (head @ listn n d @ tail))

let task = 
  textbox ~fill:Color.lightred ~tail:"goal"
let theory = 
  textbox ~width:(bp 12.) ~fill:Color.lightblue ~padding:true
    ~n:8 ~head:"theory" ~tail:"end"

let why3 =
  let th1 = theory () and th2 = theory () and th3 = theory () in
  let theories = 
    tabularl ~hpadding:(bp 5.) ~vpadding:(bp 10.)
      [[th1; empty ()];
       [empty (); th2];
       [th3; empty ()]] 
  in
  let t1 = task () and t2 = task () and t3 = task () in
  let tasks = hbox ~padding:(bp 40.) [t1; t2; t3] in
  let pr1 = tex "Alt-Ergo" and pr2 = tex "Z3" and pr3 = tex "Vampire" in
  let dr1 = empty () and dr2 = empty () and dr3 = empty () in
  let driver = 
    hbox [tex "Z3 driver"; 
	  round_rect (hbox ~padding:(bp 60.) [dr1; dr2; dr3]);
	  tex "~~~"]
  in
  let provers = (vbox ~padding:(bp 20.)  [(* pr1; *) pr2(* ; pr3 *)]) in
  let b = 
    vbox ~padding:(bp 5.)
      [hbox ~padding:(bp 25.)
	 [(* round_rect ~dx:(bp 5.) ~dy:(bp 5.) *)
	    (hbox ~padding:(bp 20.) [theories; tasks]);
	  provers];
       hbox [empty ~width:(bp 50.) (); driver]]
  in
  let arrow ?ind ?outd x y =
    Helpers.box_arrow ?ind ?outd (sub x b) (sub y b) ~sep:(bp 5.) 
  in
  let label_arrow ?ind ?outd ?pos ~label x y =
    Helpers.box_label_arrow ?ind ?outd ?pos 
      (Picture.tex label) (sub x b) (sub y b) ~sep:(bp 5.) 
  in
  let driver_arrow b1 b2 =
    let p = Point.segment 0.5 (ctr (sub b1 b)) (ctr (sub b2 b)) in
    Helpers.draw_simple_arrow ~color:Color.red ~pen 
      (Point.pt (Point.xpart p, Point.ypart (ctr (sub dr1 b)))) p
  in
  draw b ++ arrow th3 th1 ++ arrow th3 th2 ++ arrow th2 th1 ++
  arrow th2 t1 ++
  label_arrow ~pos:`Top ~label:"trans$_1$" t1 t2 ++ 
  label_arrow ~pos:`Top ~label:"trans$_2$" t2 t3 ++ 
  (* arrow t3 pr1 ++  *)arrow t3 pr2 ++ (* arrow t3 pr3 ++ *)
  driver_arrow t1 t2 ++ driver_arrow t2 t3 ++
  driver_arrow t3 pr2 ++
  nop  

let () = Metapost.emit "why3" why3
***)

let pen = Pen.transform [Transform.scaled (bp 1.5)] Pen.default

let tabular l =
  "\\begin{tabular}{c}" ^
    String.concat "\\\\\n" l ^
    "\\end{tabular}"

let rec listn n x = if n = 0 then [] else x :: listn (n-1) x

let script s = tex ~dx:zero ~dy:zero ("\\scriptsize\\tt " ^ s)

let textbox ?fill ?(width=bp 8.) ?(padding=false) ?(n=12) ?head ?tail () = 
  let head = match head with None -> [] | Some s -> [script s] in
  let tail = match tail with None -> [] | Some s -> [script s] in
  let d = rect ~pen ~dy:zero (empty ~width ~height:(bp 0.) ()) in
  let d = 
    if padding then hbox ~dx:zero ~dy:zero [empty ~width:(bp 3.) (); d] else d 
  in
  round_rect ?fill (vbox ~pos:`Left ~padding:(bp 2.) (head @ listn n d @ tail))

let task = 
  textbox ~fill:Color.lightred ~tail:"goal"
let theory = 
  textbox ~width:(bp 12.) ~fill:Color.lightblue ~padding:true
    ~n:8 ~head:"theory" ~tail:"end"

(***
let textbox ?fill ?(width=bp 8.) ?(n=12) () = 
  let d = rect ~pen ~dy:zero (empty ~width ~height:(bp 0.) ()) in
  round_rect ?fill (vbox ~padding:(bp 2.) (listn n d))

let task = textbox ~fill:Color.lightred
let theory = textbox ~fill:Color.lightblue ~n:8
***)

let th1 = theory () and th2 = theory () and th3 = theory ()

let theories = 
  tabularl ~hpadding:(bp 5.) ~vpadding:(bp 10.)
    [[th1; empty ()];
     [empty (); th2];
     [th3; empty ()]] 

let arrow b ?ind ?outd x y =
  Helpers.box_arrow ?ind ?outd (sub x b) (sub y b) ~sep:(bp 5.) 

let label_arrow b ?ind ?outd ?pos ~label x y =
  Helpers.box_label_arrow ?ind ?outd ?pos 
    (Picture.tex label) (sub x b) (sub y b) ~sep:(bp 5.) 

let pr1 = tex "Alt-Ergo" and pr2 = tex "Z3" and pr3 = tex "Vampire"
let provers = vbox ~padding:(bp 20.)  [pr1; pr2; pr3]

let why3_1 =
  let b = hbox [theories; empty ~width:(bp 120.) (); provers] in
  draw b ++ arrow b th3 th1 ++ arrow b th3 th2 ++ arrow b th2 th1 ++
  nop

let t1 = task () and t2 = task () and t3 = task ()

let why3_1 =
  let b = hbox ~padding:(bp 20.)
    [theories; empty ~width:(bp 120.) (); provers] 
  in
  draw b ++ arrow b th3 th1 ++ arrow b th3 th2 ++ arrow b th2 th1

let why3_2 =
  let b = hbox ~padding:(bp 20.)
    [theories; t1; empty ~width:(bp 80.) (); provers] 
  in
  draw b ++ arrow b th3 th1 ++ arrow b th3 th2 ++ arrow b th2 th1 ++
  arrow b th2 t1

let why3_3 =
  let b = hbox ~padding:(bp 20.)
    [theories; t1; t2; empty ~width:(bp 40.) (); provers] 
  in
  draw b ++ arrow b th3 th1 ++ arrow b th3 th2 ++ arrow b th2 th1 ++
  arrow b th2 t1 ++ label_arrow b ~pos:`Top ~label:"$T_1$" t1 t2

let why3_4 =
  let b = hbox ~padding:(bp 20.)
    [theories; t1; t2; t3; empty (); provers] 
  in
  draw b ++ arrow b th3 th1 ++ arrow b th3 th2 ++ arrow b th2 th1 ++
  arrow b th2 t1 ++ label_arrow b ~pos:`Top ~label:"$T_1$" t1 t2 ++
  label_arrow b ~pos:`Top ~label:"$T_2$" t2 t3

let why3_5 =
  let b = hbox ~padding:(bp 20.)
    [theories; t1; t2; t3; empty (); provers] 
  in
  draw b ++ arrow b th3 th1 ++ arrow b th3 th2 ++ arrow b th2 th1 ++
  arrow b th2 t1 ++ label_arrow b ~pos:`Top ~label:"$T_1$" t1 t2 ++
  label_arrow b ~pos:`Top ~label:"$T_2$" t2 t3 ++
  label_arrow b ~pos:`Top ~label:"$P$" t3 pr2

(***
  let t1 = after 2 (task ()) and t2 = after 3 (task ()) and 
      t3 = after 4 (task ()) in
  let tasks = hbox ~padding:(bp 20.) [t1; t2; t3] in
  let pr1 = tex "Alt-Ergo" and pr2 = tex "Z3" and pr3 = tex "Vampire" in
  let dr1 = empty () and dr2 = empty () and dr3 = empty () in
  let driver = after 6 
    (hbox [tex "driver"; 
	   round_rect (hbox ~padding:(bp 60.) [dr1; dr2; dr3]);
	   tex "~~~"])
  in
  let provers = after 5 (vbox ~padding:(bp 20.)  [pr1; pr2; pr3]) in
  let b = 
    vbox ~padding:(bp 5.)
      [hbox ~padding:(bp 15.)
	 [round_rect ~dx:(bp 5.) ~dy:(bp 5.)
	    (hbox ~padding:(bp 20.) [theories; tasks]);
	  provers];
       driver]
  in
  let driver_arrow b1 b2 =
    let p = Point.segment 0.5 (ctr (sub b1 b)) (ctr (sub b2 b)) in
    Helpers.draw_simple_arrow ~color:Color.red ~pen 
      (Point.pt (Point.xpart p, Point.ypart (ctr (sub dr1 b)))) p
  in
  let after j c = if i >= j then c else nop in
  draw b ++ arrow th3 th1 ++ arrow th3 th2 ++ arrow th2 th1 ++
  after 2 (arrow th2 t1) ++
  after 3 (label_arrow ~pos:`Top ~label:"$T_1$" t1 t2) ++ 
  after 4 (label_arrow ~pos:`Top ~label:"$T_2$" t2 t3) ++ 
  (if i >= 5 then arrow t3 pr1 ++ arrow t3 pr2 ++ arrow t3 pr3 else nop) ++
  (if i >= 6 then driver_arrow t1 t2 ++ driver_arrow t2 t3 ++
     driver_arrow t3 pr1 else nop) ++

let () =
  for i = 1 to 6 do Metapost.emit (Printf.sprintf "why3_%d" i) (why3 i) done

***)

let () = Metapost.emit "why3_1" why3_1
let () = Metapost.emit "why3_2" why3_2
let () = Metapost.emit "why3_3" why3_3
let () = Metapost.emit "why3_4" why3_4
let () = Metapost.emit "why3_5" why3_5

let () = exit 0


let why3_api i =
  let after j b = if i >= j then b else empty_from_box b in
  let th1 = theory () and th2 = theory () and th3 = theory () in
  let theories = 
    tabularl ~hpadding:(bp 5.) ~vpadding:(bp 10.)
      [[th1; empty ()];
       [empty (); th2];
       [th3; empty ()]] 
  in
  let t1 = task () and t2 = task () and t3 = task () in
  let tasks = hbox ~padding:(bp 20.) [t1; t2; t3] in
  let plugin s = round_rect ~fill:(Color.rgb8 255 188 43) (tex s) in
  let pluginl s = 
    round_rect ~fill:(Color.rgb8 255 188 43) 
      (vbox ~pos:`Center (List.map tex s)) 
  in
  let inputs = 
    after 2 (vbox ~padding:(bp 5.) ~pos:`Right
	       [plugin "WhyML"; plugin "TPTP"; plugin "etc."]) 
  in
  let outputs = 
    after 3 (vbox ~padding:(bp 5.) ~pos:`Left
	       [plugin "Simplify"; plugin "Alt-Ergo"; plugin "SMT-lib"; 
		plugin "etc."]) 
  in
  let transforms = 
    after 4 (hbox ~padding:(bp 5.) ~pos:`Top
	       [pluginl ["eliminate";  "algebraic"]; 
		pluginl ["encode"; "polymorphism"]; plugin "etc."]) 
  in
  let b = 
    vbox ~pos:`Left
      [tex "Your code";
       round_rect (
    vbox ~pos:`Left
      [empty ~height:(bp 30.) ();
       tex "Why3 API";
       round_rect ~dx:(bp 5.) ~dy:(bp 5.)
	 (hbox ~padding:(bp 5.) ~pos:`Top
	    [inputs;
	     vbox ~pos:`Right ~padding:(bp 5.)
	       [round_rect ~dx:(bp 5.) ~dy:(bp 5.)
		  (hbox ~padding:(bp 20.) [theories; tasks]);
		transforms];
	     outputs])
      ]
       )]
  in
  let arrow ?ind ?outd x y =
    Helpers.box_arrow ?ind ?outd (sub x b) (sub y b) ~sep:(bp 5.) 
  in
  (* let label_arrow ?ind ?outd ?pos ~label x y = *)
  (*   Helpers.box_label_arrow ?ind ?outd ?pos  *)
  (*     (Picture.tex label) (sub x b) (sub y b) ~sep:(bp 5.)  *)
  (* in *)
  (* let after j c = if i >= j then c else nop in *)
  draw b ++ arrow th3 th1 ++ arrow th3 th2 ++ arrow th2 th1 ++
  arrow th2 t1 ++ arrow t1 t2 ++ arrow t2 t3 ++ 
  nop  

let () = 
  for i = 1 to 4 do
    Metapost.emit (Printf.sprintf "why3_api_%d" i) (why3_api i)
  done

(*
Local Variables: 
compile-command: "mlpost -native -latex why3.tex -xpdf figures.ml"
End: 
*)

